@@ -1144,11 +1144,15 @@ def set_sum(set_iterable):
     return reduce(or_, set_iterable, set())
 
 
-def div_ceil(nr, dr):
+def div_ceil(nr: int, dr: int):
     return -(-nr // dr)
 
 
-def uniform_interval_splitting(n, granularity, max_intervals):
+def uniform_interval_splitting(
+            n: int,
+            granularity: int,
+            max_intervals: int
+        ) -> tuple[int, int]:
     """ Return *(interval_size, num_intervals)* such that::
 
         num_intervals * interval_size >= n
@@ -3112,6 +3116,22 @@ def opt_frozen_dataclass(
     return map_cls
 
 
+ShapeT = TypeVar("ShapeT", bound=tuple[int, ...])
+
+
+def ndindex(shape: ShapeT) -> Iterable[ShapeT]:
+    """A more-precisely-typed wrapper around :class:`numpy.ndindex`, since
+    :mod:`numpy` 2.3.1 only provides a tuple of :class:`~typing.Any`.
+
+    .. note:: Requires :mod:`numpy`.
+
+    .. versionadded:: 2025.2
+    """
+    # undocumented for now
+    import numpy as np
+    return np.ndindex(shape)
+
+
 def _test():
     import doctest
     doctest.testmod()
